from .api import (
    ConsumeOutputEvent,
    CreatePipeline,
    CreateSpace,
    EventContext,
    FunctionEnvironments,
    FunctionLogEntry,
    FunctionLogs,
    GetDetailedSpacePipeline,
    PipelineState,
    SeverityCodeInput,
    SinkConnector,
    SourceConnector,
    SpacePipeline,
    SpaceScope,
    UpdatePipeline,
)

__all__ = [
    "CreatePipeline",
    "FunctionEnvironments",
    "FunctionLogEntry",
    "FunctionLogs",
    "GetDetailedSpacePipeline",
    "PipelineState",
    "SeverityCodeInput",
    "SinkConnector",
    "SourceConnector",
    "SpacePipeline",
    "UpdatePipeline",
    "SpaceScope",
    "CreateSpace",
    "EventContext",
    "ConsumeOutputEvent",
]



from .access_token import ListAccessTokensRequest, StatusAccessTokenRequest
from .artifact import (
    GetArtifactRequest,
    PostArtifactRequest,
)
from .base import (
    BaseManagementRequest,
    BasePipelineManagementRequest,
    BaseRequest,
    BaseResponse,
    BaseSpaceManagementDataRequest,
)
from .consumeevent import (
    ConsumeEventRequest,
    ConsumeEventResponse,
    ConsumeEventResponseBody,
)
from .consumefailed import (
    ConsumeFailedRequest,
    ConsumeFailedResponse,
    ConsumeFailedResponseBody,
)
from .function import (
    FetchFunctionRequest,
    GetFunctionLogsRequest,
    GetFunctionLogsResponse,
    TestFunctionRequest,
    TestFunctionResponse,
    UpdateFunctionRequest,
)
from .pipeline import (
    CreatePipelineRequest,
    CreatePipelineResponse,
    DeletePipelineRequest,
    GetPipelineRequest,
    GetPipelineResponse,
    ListPipelinesRequest,
    ListPipelinesResponse,
    UpdatePipelineRequest,
    UpdatePipelineResponse,
)
from .publishevent import (
    PublishEventRequest,
    PublishEventRequestBody,
    PublishEventResponse,
    PublishEventResponseBody,
)
from .space import (
    CreateSpaceRequest,
    CreateSpaceResponse,
    DeleteSpaceRequest,
    ListSpacesRequest,
    ListSpacesResponse,
)

__all__ = [
    "BaseManagementRequest",
    "BasePipelineManagementRequest",
    "BaseRequest",
    "BaseResponse",
    "BaseSpaceManagementDataRequest",
    "ConsumeEventRequest",
    "ConsumeEventResponse",
    "ConsumeEventResponseBody",
    "ConsumeFailedRequest",
    "ConsumeFailedResponse",
    "ConsumeFailedResponseBody",
    "CreatePipelineRequest",
    "CreatePipelineResponse",
    "DeletePipelineRequest",
    "DeleteSpaceRequest",
    "GetPipelineRequest",
    "GetPipelineResponse",
    "ListPipelinesRequest",
    "ListPipelinesResponse",
    "ListAccessTokensRequest",
    "PublishEventRequest",
    "PublishEventRequestBody",
    "PublishEventResponse",
    "PublishEventResponseBody",
    "GetArtifactRequest",
    "GetFunctionLogsRequest",
    "GetFunctionLogsResponse",
    "StatusAccessTokenRequest",
    "ListSpacesResponse",
    "ListSpacesRequest",
    "CreateSpaceRequest",
    "CreateSpaceResponse",
    "UpdatePipelineRequest",
    "UpdatePipelineResponse",
    "UpdateFunctionRequest",
    "FetchFunctionRequest",
    "PostArtifactRequest",
    "TestFunctionRequest",
    "TestFunctionResponse",
]



from __future__ import annotations

import dataclasses

from dataclasses_json import Undefined, dataclass_json

from glassflow import utils


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclasses.dataclass
class Error(Exception):
    """Bad request error response

    Attributes:
        message: A message describing the error

    """

    detail: str = dataclasses.field(
        metadata={"dataclasses_json": {"letter_case": utils.get_field_name("detail")}}
    )

    def __str__(self) -> str:
        return utils.marshal_json(self, type(self))



import random
import time
from typing import Optional

from .api_client import APIClient
from .models import errors, operations
from .models.operations.base import BasePipelineDataRequest, BaseResponse
from .utils import utils


class PipelineDataClient(APIClient):
    """Base Client object to publish and consume events from the given pipeline.

    Attributes:
        glassflow_config: GlassFlowConfig object to interact with GlassFlow API
        pipeline_id: The pipeline id to interact with
        pipeline_access_token: The access token to access the pipeline
    """

    def __init__(self, pipeline_id: str, pipeline_access_token: str):
        super().__init__()
        self.pipeline_id = pipeline_id
        self.pipeline_access_token = pipeline_access_token

    def validate_credentials(self) -> None:
        """
        Check if the pipeline credentials are valid and raise an error if not
        """
        request = operations.StatusAccessTokenRequest(
            pipeline_id=self.pipeline_id,
            x_pipeline_access_token=self.pipeline_access_token,
        )
        self._request(
            method="GET",
            endpoint="/pipelines/{pipeline_id}/status/access_token",
            request=request,
        )

    def _request(
        self, method: str, endpoint: str, request: BasePipelineDataRequest, **kwargs
    ) -> BaseResponse:
        try:
            res = super()._request(method, endpoint, request, **kwargs)
        except errors.ClientError as e:
            if e.status_code == 401:
                raise errors.PipelineAccessTokenInvalidError(e.raw_response) from e
            elif e.status_code == 404:
                raise errors.PipelineNotFoundError(
                    self.pipeline_id, e.raw_response
                ) from e
            else:
                raise e
        return res


class PipelineDataSource(PipelineDataClient):
    def publish(self, request_body: dict) -> operations.PublishEventResponse:
        """Push a new message into the pipeline

        Args:
            request_body: The message to be published into the pipeline

        Returns:
            PublishEventResponse: Response object containing the status
                code and the raw response

        Raises:
            ClientError: If an error occurred while publishing the event
        """
        request = operations.PublishEventRequest(
            pipeline_id=self.pipeline_id,
            x_pipeline_access_token=self.pipeline_access_token,
            request_body=request_body,
        )
        base_res = self._request(
            method="POST",
            endpoint="/pipelines/{pipeline_id}/topics/input/events",
            request=request,
        )

        return operations.PublishEventResponse(
            status_code=base_res.status_code,
            content_type=base_res.content_type,
            raw_response=base_res.raw_response,
        )


class PipelineDataSink(PipelineDataClient):
    def __init__(self, pipeline_id: str, pipeline_access_token: str):
        super().__init__(pipeline_id, pipeline_access_token)

        # retry delay for consuming messages (in seconds)
        self._consume_retry_delay_minimum = 1
        self._consume_retry_delay_current = 1
        self._consume_retry_delay_max = 60

    def consume(self) -> operations.ConsumeEventResponse:
        """Consume the last message from the pipeline

        Returns:
            ConsumeEventResponse: Response object containing the status
                code and the raw response

        Raises:
            ClientError: If an error occurred while consuming the event

        """
        request = operations.ConsumeEventRequest(
            pipeline_id=self.pipeline_id,
            x_pipeline_access_token=self.pipeline_access_token,
        )

        self._respect_retry_delay()
        base_res = self._request(
            method="POST",
            endpoint="/pipelines/{pipeline_id}/topics/output/events/consume",
            request=request,
        )

        res = operations.ConsumeEventResponse(
            status_code=base_res.status_code,
            content_type=base_res.content_type,
            raw_response=base_res.raw_response,
        )

        self._update_retry_delay(base_res.status_code)
        if res.status_code == 200:
            if not utils.match_content_type(res.content_type, "application/json"):
                raise errors.UnknownContentTypeError(res.raw_response)

            self._consume_retry_delay_current = self._consume_retry_delay_minimum
            body = utils.unmarshal_json(
                res.raw_response.text, Optional[operations.ConsumeEventResponseBody]
            )
            res.body = body
        elif res.status_code == 204:
            # No messages to be consumed.
            # update the retry delay
            # Return an empty response body
            body = operations.ConsumeEventResponseBody("", "", {})
            res.body = body
        elif res.status_code == 429:
            # update the retry delay
            body = operations.ConsumeEventResponseBody("", "", {})
            res.body = body
        elif not utils.match_content_type(res.content_type, "application/json"):
            raise errors.UnknownContentTypeError(res.raw_response)

        return res

    def consume_failed(self) -> operations.ConsumeFailedResponse:
        """Consume the failed message from the pipeline

        Returns:
            ConsumeFailedResponse: Response object containing the status
                code and the raw response

        Raises:
            ClientError: If an error occurred while consuming the event

        """
        request = operations.ConsumeFailedRequest(
            pipeline_id=self.pipeline_id,
            x_pipeline_access_token=self.pipeline_access_token,
        )

        self._respect_retry_delay()
        base_res = self._request(
            method="POST",
            endpoint="/pipelines/{pipeline_id}/topics/failed/events/consume",
            request=request,
        )

        res = operations.ConsumeFailedResponse(
            status_code=base_res.status_code,
            content_type=base_res.content_type,
            raw_response=base_res.raw_response,
        )

        self._update_retry_delay(res.status_code)
        if res.status_code == 200:
            if not utils.match_content_type(res.content_type, "application/json"):
                raise errors.UnknownContentTypeError(res.raw_response)

            self._consume_retry_delay_current = self._consume_retry_delay_minimum
            body = utils.unmarshal_json(
                res.raw_response.text, Optional[operations.ConsumeFailedResponseBody]
            )
            res.body = body
        elif res.status_code == 204:
            # No messages to be consumed. Return an empty response body
            body = operations.ConsumeFailedResponseBody("", "", {})
            res.body = body
        elif res.status_code == 429:
            # update the retry delay
            body = operations.ConsumeEventResponseBody("", "", {})
            res.body = body
        elif not utils.match_content_type(res.content_type, "application/json"):
            raise errors.UnknownContentTypeError(res.raw_response)
        return res

    def _update_retry_delay(self, status_code: int):
        if status_code == 200:
            self._consume_retry_delay_current = self._consume_retry_delay_minimum
        elif status_code == 204 or status_code == 429:
            self._consume_retry_delay_current *= 2
            self._consume_retry_delay_current = min(
                self._consume_retry_delay_current, self._consume_retry_delay_max
            )
            self._consume_retry_delay_current += random.uniform(0, 0.1)

    def _respect_retry_delay(self):
        if self._consume_retry_delay_current > self._consume_retry_delay_minimum:
            # sleep before making the request
            time.sleep(self._consume_retry_delay_current)



from __future__ import annotations

import dataclasses

from ...utils import generate_metadata_for_query_parameters
from ..api import (
    ConsumeOutputEvent,
    FunctionEnvironments,
    FunctionLogs,
    SeverityCodeInput,
)
from .base import BasePipelineManagementRequest, BaseResponse


@dataclasses.dataclass
class GetFunctionLogsRequest(BasePipelineManagementRequest):
    page_size: int = dataclasses.field(
        default=50,
        metadata=generate_metadata_for_query_parameters("page_size"),
    )
    page_token: str = dataclasses.field(
        default=None,
        metadata=generate_metadata_for_query_parameters("page_token"),
    )
    severity_code: SeverityCodeInput | None = dataclasses.field(
        default=None,
        metadata=generate_metadata_for_query_parameters("severity_code"),
    )
    start_time: str | None = dataclasses.field(
        default=None,
        metadata=generate_metadata_for_query_parameters("start_time"),
    )
    end_time: str | None = dataclasses.field(
        default=None,
        metadata=generate_metadata_for_query_parameters("end_time"),
    )


@dataclasses.dataclass
class GetFunctionLogsResponse(BaseResponse):
    logs: FunctionLogs
    next: str


@dataclasses.dataclass
class FetchFunctionRequest(BasePipelineManagementRequest):
    pass


@dataclasses.dataclass
class UpdateFunctionRequest(BasePipelineManagementRequest):
    environments: FunctionEnvironments | None = dataclasses.field(default=None)


@dataclasses.dataclass
class TestFunctionRequest(BasePipelineManagementRequest):
    request_body: dict = dataclasses.field(
        default=None, metadata={"request": {"media_type": "application/json"}}
    )


@dataclasses.dataclass
class TestFunctionResponse(ConsumeOutputEvent, BaseResponse):
    pass



"""Dataclasses for the consume event operation"""

from __future__ import annotations

import dataclasses

from dataclasses_json import config, dataclass_json

from .base import BasePipelineDataRequest, BaseResponse


@dataclasses.dataclass
class ConsumeEventRequest(BasePipelineDataRequest):
    """Request to consume an event from a pipeline topic

    Attributes:
        pipeline_id: The id of the pipeline
        organization_id: The id of the organization
        x_pipeline_access_token: The access token of the pipeline

    """

    pass


@dataclass_json
@dataclasses.dataclass
class ConsumeEventResponseBody:
    """Event response body after transformation

    Attributes:
        req_id: The request id
        receive_time: The time when the event was received
        event: The event received

    """

    req_id: str = dataclasses.field()
    receive_time: str = dataclasses.field()
    event: dict = dataclasses.field(metadata=config(field_name="response"))


@dataclasses.dataclass
class ConsumeEventResponse(BaseResponse):
    """Response to consume an event from a pipeline topic

    Attributes:
        content_type: HTTP response content type for this operation
        status_code: HTTP response status code for this operation
        raw_response: Raw HTTP response; suitable for custom response parsing
        body: the response body from the api call

    """

    body: ConsumeEventResponseBody | None = dataclasses.field(default=None)

    def json(self) -> dict:
        """Return the response body as a JSON object.
        This method is to have compatibility with the requests.Response.json() method

        Returns:
            dict: The transformed event as a JSON object
        """
        return self.body.event



from .clienterror import (
    ClientError,
    PipelineAccessTokenInvalidError,
    PipelineNotFoundError,
    SpaceIsNotEmptyError,
    SpaceNotFoundError,
    UnauthorizedError,
    UnknownContentTypeError,
)
from .error import Error

__all__ = [
    "Error",
    "ClientError",
    "PipelineNotFoundError",
    "PipelineAccessTokenInvalidError",
    "SpaceNotFoundError",
    "UnknownContentTypeError",
    "UnauthorizedError",
    "SpaceIsNotEmptyError",
]



from __future__ import annotations

import dataclasses
from enum import Enum

from ...utils import generate_metadata_for_query_parameters
from ..api import CreateSpace, SpaceScope
from .base import BaseManagementRequest, BaseResponse, BaseSpaceManagementDataRequest


@dataclasses.dataclass
class ListSpacesResponse(BaseResponse):
    total_amount: int
    spaces: list[SpaceScope]


class Order(str, Enum):
    asc = "asc"
    desc = "desc"


@dataclasses.dataclass
class ListSpacesRequest(BaseManagementRequest):
    page_size: int = dataclasses.field(
        default=50,
        metadata=generate_metadata_for_query_parameters("page_size"),
    )
    page: int = dataclasses.field(
        default=1,
        metadata=generate_metadata_for_query_parameters("page"),
    )
    order_by: Order = dataclasses.field(
        default=Order.asc,
        metadata=generate_metadata_for_query_parameters("order_by"),
    )


@dataclasses.dataclass
class CreateSpaceRequest(BaseManagementRequest, CreateSpace):
    pass


@dataclasses.dataclass
class CreateSpaceResponse(BaseResponse):
    name: str
    id: str
    created_at: str


@dataclasses.dataclass
class DeleteSpaceRequest(BaseSpaceManagementDataRequest):
    pass



from __future__ import annotations

from .client import APIClient
from .models import api, errors, operations


class Space(APIClient):
    def __init__(
        self,
        personal_access_token: str,
        name: str | None = None,
        id: str | None = None,
        created_at: str | None = None,
        organization_id: str | None = None,
    ):
        """Creates a new GlassFlow pipeline object

        Args:
            personal_access_token: The personal access token to authenticate
                against GlassFlow
            name: Name of the space
            id: ID of the GlassFlow Space you want to create the pipeline in
            created_at: Timestamp when the space was created

        """
        super().__init__()
        self.name = name
        self.id = id
        self.created_at = created_at
        self.organization_id = organization_id
        self.personal_access_token = personal_access_token

    def create(self) -> Space:
        """
        Creates a new GlassFlow space

        Returns:
            self: Space object

        Raises:
            ValueError: If name is not provided in the constructor

        """
        if self.name is None:
            raise ValueError("Name must be provided in order to create the space")
        create_space = api.CreateSpace(name=self.name)
        request = operations.CreateSpaceRequest(
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
            **create_space.__dict__,
        )
        base_res = self._request(method="POST", endpoint="/spaces", request=request)

        res = operations.CreateSpaceResponse(
            status_code=base_res.status_code,
            content_type=base_res.content_type,
            raw_response=base_res.raw_response,
            **base_res.raw_response.json(),
        )

        self.id = res.id
        self.created_at = res.created_at
        self.name = res.name
        return self

    def delete(self) -> None:
        """
        Deletes a GlassFlow space

        Returns:

        Raises:
            ValueError: If ID is not provided in the constructor
            SpaceNotFoundError: If ID provided does not match any
                existing space in GlassFlow
            UnauthorizedError: If the Personal Access Token is not
                provided or is invalid
        """
        if self.id is None:
            raise ValueError("Space id must be provided in the constructor")

        request = operations.DeleteSpaceRequest(
            space_id=self.id,
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
        )
        self._request(
            method="DELETE",
            endpoint=f"/spaces/{self.id}",
            request=request,
        )

    def _request(
        self,
        method: str,
        endpoint: str,
        request: operations.BaseManagementRequest,
        **kwargs,
    ) -> operations.BaseResponse:
        try:
            return super()._request(
                method=method, endpoint=endpoint, request=request, **kwargs
            )
        except errors.ClientError as e:
            if e.status_code == 404:
                raise errors.SpaceNotFoundError(self.id, e.raw_response) from e
            elif e.status_code == 401:
                raise errors.UnauthorizedError(e.raw_response) from e
            elif e.status_code == 409:
                raise errors.SpaceIsNotEmptyError(e.raw_response) from e
            else:
                raise e



import dataclasses
from typing import Optional

from requests import Response

from ...utils import generate_metadata_for_query_parameters


@dataclasses.dataclass()
class BaseRequest:
    pass


@dataclasses.dataclass()
class BaseManagementRequest(BaseRequest):
    organization_id: Optional[str] = dataclasses.field(
        default=None,
        metadata=generate_metadata_for_query_parameters("organization_id"),
    )
    personal_access_token: str = dataclasses.field(
        default=None,
        metadata={
            "header": {
                "field_name": "Personal-Access-Token",
                "style": "simple",
                "explode": False,
            }
        },
    )


@dataclasses.dataclass
class BasePipelineRequest(BaseRequest):
    pipeline_id: str = dataclasses.field(
        metadata={
            "path_param": {
                "field_name": "pipeline_id",
                "style": "simple",
                "explode": False,
            }
        }
    )
    organization_id: Optional[str] = dataclasses.field(
        default=None,
        metadata=generate_metadata_for_query_parameters("organization_id"),
    )


@dataclasses.dataclass
class BasePipelineDataRequest(BasePipelineRequest):
    x_pipeline_access_token: str = dataclasses.field(
        default=None,
        metadata={
            "header": {
                "field_name": "X-PIPELINE-ACCESS-TOKEN",
                "style": "simple",
                "explode": False,
            }
        },
    )


@dataclasses.dataclass
class BaseSpaceRequest(BaseRequest):
    space_id: str


@dataclasses.dataclass
class BasePipelineManagementRequest(BaseManagementRequest, BasePipelineRequest):
    pass


@dataclasses.dataclass
class BaseSpaceManagementDataRequest(BaseManagementRequest, BaseSpaceRequest):
    pass


@dataclasses.dataclass
class BaseResponse:
    content_type: str = dataclasses.field()
    status_code: int = dataclasses.field()
    raw_response: Response = dataclasses.field()



from __future__ import annotations

import sys

import requests as requests_http

from .config import GlassFlowConfig
from .models import errors
from .models.operations.base import BaseRequest, BaseResponse
from .utils import utils as utils


class APIClient:
    glassflow_config = GlassFlowConfig()

    def __init__(self):
        super().__init__()
        self.client = requests_http.Session()

    def _get_headers(
        self, request: BaseRequest, req_content_type: str | None = None
    ) -> dict:
        headers = utils.get_req_specific_headers(request)
        headers["Accept"] = "application/json"
        headers["Gf-Client"] = self.glassflow_config.glassflow_client
        headers["User-Agent"] = self.glassflow_config.user_agent
        headers["Gf-Python-Version"] = (
            f"{sys.version_info.major}."
            f"{sys.version_info.minor}."
            f"{sys.version_info.micro}"
        )

        if req_content_type and req_content_type not in (
            "multipart/form-data",
            "multipart/mixed",
        ):
            headers["content-type"] = req_content_type

        return headers

    def _request(
        self,
        method: str,
        endpoint: str,
        request: BaseRequest,
        serialization_method: str = "json",
    ) -> BaseResponse:
        request_type = type(request)

        url = utils.generate_url(
            request_type,
            self.glassflow_config.server_url,
            endpoint,
            request,
        )

        req_content_type, data, form = utils.serialize_request_body(
            request=request,
            request_type=request_type,
            request_field_name="request_body",
            nullable=False,
            optional=True,
            serialization_method=serialization_method,
        )
        if method == "GET":
            data = None

        headers = self._get_headers(request, req_content_type)
        query_params = utils.get_query_params(request_type, request)

        # make the request
        http_res = self.client.request(
            method, url=url, params=query_params, headers=headers, data=data, files=form
        )
        content_type = http_res.headers.get("Content-Type")

        res = BaseResponse(
            status_code=http_res.status_code,
            content_type=content_type,
            raw_response=http_res,
        )

        if http_res.status_code in [400, 500]:
            out = utils.unmarshal_json(http_res.text, errors.Error)
            out.raw_response = http_res
            raise out
        elif http_res.status_code == 429:
            pass
        elif 400 < http_res.status_code < 600:
            raise errors.ClientError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )

        return res



from __future__ import annotations

from .client import APIClient
from .models import api, errors, operations
from .pipeline_data import PipelineDataSink, PipelineDataSource


class Pipeline(APIClient):
    def __init__(
        self,
        personal_access_token: str,
        name: str | None = None,
        space_id: str | None = None,
        id: str | None = None,
        source_kind: str | None = None,
        source_config: dict | None = None,
        sink_kind: str | None = None,
        sink_config: dict | None = None,
        requirements: str | None = None,
        transformation_file: str | None = None,
        env_vars: list[dict[str, str]] | None = None,
        state: api.PipelineState = "running",
        organization_id: str | None = None,
        metadata: dict | None = None,
        created_at: str | None = None,
    ):
        """Creates a new GlassFlow pipeline object

        Args:
            personal_access_token: The personal access token to authenticate
                against GlassFlow
            id: Pipeline ID
            name: Name of the pipeline
            space_id: ID of the GlassFlow Space you want to create the pipeline in
            transformation_file: Path to file with transformation function of
                the pipeline.
            requirements: Requirements.txt of the pipeline
            source_kind: Kind of source for the pipeline. If no source is
                provided, the default source will be SDK
            source_config: Configuration of the pipeline's source
            sink_kind: Kind of sink for the pipeline. If no sink is provided,
                the default sink will be SDK
            sink_config: Configuration of the pipeline's sink
            env_vars: Environment variables to pass to the pipeline
            state: State of the pipeline after creation.
                It can be either "running" or "paused"
            metadata: Metadata of the pipeline
            created_at: Timestamp when the pipeline was created

        Raises:
            FailNotFoundError: If the transformation file is provided and
                does not exist
        """
        super().__init__()
        self.id = id
        self.name = name
        self.space_id = space_id
        self.personal_access_token = personal_access_token
        self.source_kind = source_kind
        self.source_config = source_config
        self.sink_kind = sink_kind
        self.sink_config = sink_config
        self.requirements = requirements
        self.transformation_code = None
        self.transformation_file = transformation_file
        self.env_vars = env_vars
        self.state = state
        self.organization_id = organization_id
        self.metadata = metadata if metadata is not None else {}
        self.created_at = created_at
        self.access_tokens = []

        if self.transformation_file is not None:
            self._read_transformation_file()

        if source_kind is not None and self.source_config is not None:
            self.source_connector = dict(
                kind=self.source_kind,
                config=self.source_config,
            )
        elif self.source_kind is None and self.source_config is None:
            self.source_connector = None
        else:
            raise ValueError("Both source_kind and source_config must be provided")

        if self.sink_kind is not None and self.sink_config is not None:
            self.sink_connector = dict(
                kind=sink_kind,
                config=sink_config,
            )
        elif self.sink_kind is None and self.sink_config is None:
            self.sink_connector = None
        else:
            raise ValueError("Both sink_kind and sink_config must be provided")

    def fetch(self) -> Pipeline:
        """
        Fetches pipeline information from the GlassFlow API

        Returns:
            self: Pipeline object

        Raises:
            ValueError: If ID is not provided in the constructor
            PipelineNotFoundError: If ID provided does not match any
                existing pipeline in GlassFlow
            UnauthorizedError: If the Personal Access Token is not
                provider or is invalid
        """
        if self.id is None:
            raise ValueError(
                "Pipeline id must be provided in order to fetch it's details"
            )

        request = operations.GetPipelineRequest(
            pipeline_id=self.id,
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
        )

        base_res = self._request(
            method="GET",
            endpoint=f"/pipelines/{self.id}",
            request=request,
        )
        self._fill_pipeline_details(base_res.raw_response.json())

        # Fetch Pipeline Access Tokens
        self._list_access_tokens()

        # Fetch function source
        self._get_function_artifact()

        return self

    def create(self) -> Pipeline:
        """
        Creates a new GlassFlow pipeline

        Returns:
            self: Pipeline object

        Raises:
            ValueError: If name is not provided in the constructor
            ValueError: If space_id is not provided in the constructor
            ValueError: If transformation_file is not provided
                in the constructor
        """
        create_pipeline = api.CreatePipeline(
            name=self.name,
            space_id=self.space_id,
            transformation_function=self.transformation_code,
            requirements_txt=self.requirements,
            source_connector=self.source_connector,
            sink_connector=self.sink_connector,
            environments=self.env_vars,
            state=self.state,
            metadata=self.metadata,
        )
        if self.name is None:
            raise ValueError("Name must be provided in order to create the pipeline")
        if self.space_id is None:
            raise ValueError("Argument space_id must be provided in the constructor")
        if self.transformation_file is None:
            raise ValueError(
                "Argument transformation_file must be provided in the constructor"
            )
        else:
            self._read_transformation_file()

        request = operations.CreatePipelineRequest(
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
            **create_pipeline.__dict__,
        )

        base_res = self._request(method="POST", endpoint="/pipelines", request=request)
        res = operations.CreatePipelineResponse(
            status_code=base_res.status_code,
            content_type=base_res.content_type,
            raw_response=base_res.raw_response,
            **base_res.raw_response.json(),
        )

        self.id = res.id
        self.created_at = res.created_at
        self.access_tokens.append({"name": "default", "token": res.access_token})
        return self

    def update(
        self,
        name: str | None = None,
        state: api.PipelineState | None = None,
        transformation_file: str | None = None,
        requirements: str | None = None,
        metadata: dict | None = None,
        source_kind: str | None = None,
        source_config: dict | None = None,
        sink_kind: str | None = None,
        sink_config: dict | None = None,
        env_vars: list[dict[str, str]] | None = None,
    ) -> Pipeline:
        """
        Updates a GlassFlow pipeline

        Args:

            name: Name of the pipeline
            state: State of the pipeline after creation.
                It can be either "running" or "paused"
            transformation_file: Path to file with transformation function of
                the pipeline.
            requirements: Requirements.txt of the pipeline
            source_kind: Kind of source for the pipeline. If no source is
                provided, the default source will be SDK
            source_config: Configuration of the pipeline's source
            sink_kind: Kind of sink for the pipeline. If no sink is provided,
                the default sink will be SDK
            sink_config: Configuration of the pipeline's sink
            env_vars: Environment variables to pass to the pipeline
            metadata: Metadata of the pipeline

        Returns:
            self: Updated pipeline

        """

        # Fetch current pipeline data
        self.fetch()

        if transformation_file is not None or requirements is not None:
            if transformation_file is not None:
                with open(transformation_file) as f:
                    file = f.read()
            else:
                file = self.transformation_code

            if requirements is None:
                requirements = self.requirements

            self._upload_function_artifact(file, requirements)
            self.requirements = requirements
            self.transformation_code = file

        if source_kind is not None:
            source_connector = dict(
                kind=source_kind,
                config=source_config,
            )
        else:
            source_connector = self.source_connector

        if sink_kind is not None:
            sink_connector = dict(
                kind=sink_kind,
                config=sink_config,
            )
        else:
            sink_connector = self.sink_connector

        if env_vars is not None:
            self._update_function(env_vars)

        request = operations.UpdatePipelineRequest(
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
            name=name if name is not None else self.name,
            state=state if state is not None else self.state,
            metadata=metadata if metadata is not None else self.metadata,
            source_connector=source_connector,
            sink_connector=sink_connector,
        )

        base_res = self._request(
            method="PATCH", endpoint=f"/pipelines/{self.id}", request=request
        )
        self._fill_pipeline_details(base_res.raw_response.json())
        return self

    def delete(self) -> None:
        """
        Deletes a GlassFlow pipeline

        Returns:

        Raises:
            ValueError: If ID is not provided in the constructor
            PipelineNotFoundError: If ID provided does not match any
                existing pipeline in GlassFlow
            UnauthorizedError: If the Personal Access Token is not
                provided or is invalid
        """
        if self.id is None:
            raise ValueError("Pipeline id must be provided")

        request = operations.DeletePipelineRequest(
            pipeline_id=self.id,
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
        )
        self._request(
            method="DELETE",
            endpoint=f"/pipelines/{self.id}",
            request=request,
        )

    def get_logs(
        self,
        page_size: int = 50,
        page_token: str | None = None,
        severity_code: api.SeverityCodeInput = api.SeverityCodeInput.integer_100,
        start_time: str | None = None,
        end_time: str | None = None,
    ) -> operations.GetFunctionLogsResponse:
        """
        Get the pipeline's logs

        Args:
            page_size: Pagination size
            page_token: Page token filter (use for pagination)
            severity_code: Severity code filter
            start_time: Start time filter
            end_time: End time filter

        Returns:
            PipelineFunctionsGetLogsResponse: Response with the logs
        """
        request = operations.GetFunctionLogsRequest(
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
            pipeline_id=self.id,
            page_size=page_size,
            page_token=page_token,
            severity_code=severity_code,
            start_time=start_time,
            end_time=end_time,
        )
        base_res = self._request(
            method="GET",
            endpoint=f"/pipelines/{self.id}/functions/main/logs",
            request=request,
        )
        base_res_json = base_res.raw_response.json()
        logs = [
            api.FunctionLogEntry.from_dict(entry) for entry in base_res_json["logs"]
        ]
        return operations.GetFunctionLogsResponse(
            status_code=base_res.status_code,
            content_type=base_res.content_type,
            raw_response=base_res.raw_response,
            logs=logs,
            next=base_res_json["next"],
        )

    def _list_access_tokens(self) -> Pipeline:
        request = operations.ListAccessTokensRequest(
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
            pipeline_id=self.id,
        )
        base_res = self._request(
            method="GET",
            endpoint=f"/pipelines/{self.id}/access_tokens",
            request=request,
        )
        res_json = base_res.raw_response.json()
        self.access_tokens = res_json["access_tokens"]
        return self

    def _get_function_artifact(self) -> Pipeline:
        """
        Fetch pipeline function source

        Returns:
            self: Pipeline with function source details
        """
        request = operations.GetArtifactRequest(
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
            pipeline_id=self.id,
        )
        base_res = self._request(
            method="GET",
            endpoint=f"/pipelines/{self.id}/functions/main/artifacts/latest",
            request=request,
        )
        res_json = base_res.raw_response.json()
        self.transformation_code = res_json["transformation_function"]

        if "requirements_txt" in res_json:
            self.requirements = res_json["requirements_txt"]
        return self

    def _upload_function_artifact(self, file: str, requirements: str) -> None:
        request = operations.PostArtifactRequest(
            pipeline_id=self.id,
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
            file=file,
            requirementsTxt=requirements,
        )
        try:
            self._request(
                method="POST",
                endpoint=f"/pipelines/{self.id}/functions/main/artifacts",
                request=request,
                serialization_method="multipart",
            )
        except errors.ClientError as e:
            if e.status_code == 425:
                # TODO: Figure out appropriate behaviour
                print("Update still in progress")
            else:
                raise e

    def _update_function(self, env_vars):
        """
        Patch pipeline function

        Args:
            env_vars: Environment variables to update

        Returns:
            self: Pipeline with updated function
        """
        request = operations.UpdateFunctionRequest(
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
            pipeline_id=self.id,
            environments=env_vars,
        )
        base_res = self._request(
            method="PATCH",
            endpoint=f"/pipelines/{self.id}/functions/main",
            request=request,
        )
        res_json = base_res.raw_response.json()
        self.env_vars = res_json["environments"]
        return self

    def get_source(
        self, pipeline_access_token_name: str | None = None
    ) -> PipelineDataSource:
        """
        Get source client to publish data to the pipeline

        Args:
            pipeline_access_token_name (str | None): Name of the pipeline
                access token to use. If not specified, the default token
                will be used

        Returns:
            PipelineDataSource: Source client to publish data to the pipeline

        Raises:
            ValueError: If pipeline id is not provided in the constructor
        """
        return self._get_data_client("source", pipeline_access_token_name)

    def get_sink(
        self, pipeline_access_token_name: str | None = None
    ) -> PipelineDataSink:
        """
        Get sink client to consume data from the pipeline

        Args:
            pipeline_access_token_name (str | None): Name of the pipeline
                access token to use. If not specified, the default token
                will be used

        Returns:
            PipelineDataSink: Sink client to consume data from the pipeline

        Raises:
            ValueError: If pipeline id is not provided in the constructor
        """
        return self._get_data_client("sink", pipeline_access_token_name)

    def _get_data_client(
        self, client_type: str, pipeline_access_token_name: str | None = None
    ) -> PipelineDataSource | PipelineDataSink:
        if self.id is None:
            raise ValueError("Pipeline id must be provided in the constructor")
        elif len(self.access_tokens) == 0:
            self._list_access_tokens()

        if pipeline_access_token_name is not None:
            for t in self.access_tokens:
                if t["name"] == pipeline_access_token_name:
                    token = t["token"]
                    break
            else:
                raise ValueError(
                    f"Token with name {pipeline_access_token_name} " f"was not found"
                )
        else:
            token = self.access_tokens[0]["token"]
        if client_type == "source":
            client = PipelineDataSource(
                pipeline_id=self.id,
                pipeline_access_token=token,
            )
        elif client_type == "sink":
            client = PipelineDataSink(
                pipeline_id=self.id,
                pipeline_access_token=token,
            )
        else:
            raise ValueError("client_type must be either source or sink")
        return client

    def _request(
        self,
        method: str,
        endpoint: str,
        request: operations.BaseManagementRequest,
        **kwargs,
    ) -> operations.BaseResponse:
        try:
            return super()._request(
                method=method, endpoint=endpoint, request=request, **kwargs
            )
        except errors.ClientError as e:
            if e.status_code == 404:
                raise errors.PipelineNotFoundError(self.id, e.raw_response) from e
            elif e.status_code == 401:
                raise errors.UnauthorizedError(e.raw_response) from e
            else:
                raise e

    def _read_transformation_file(self):
        try:
            with open(self.transformation_file) as f:
                self.transformation_code = f.read()
        except FileNotFoundError:
            raise

    def _fill_pipeline_details(self, pipeline_details: dict) -> Pipeline:
        self.id = pipeline_details["id"]
        self.name = pipeline_details["name"]
        self.space_id = pipeline_details["space_id"]
        self.state = pipeline_details["state"]
        if pipeline_details["source_connector"]:
            self.source_kind = pipeline_details["source_connector"]["kind"]
            self.source_config = pipeline_details["source_connector"]["config"]
        if pipeline_details["sink_connector"]:
            self.sink_kind = pipeline_details["sink_connector"]["kind"]
            self.sink_config = pipeline_details["sink_connector"]["config"]
        self.created_at = pipeline_details["created_at"]
        self.env_vars = pipeline_details["environments"]

        return self

    def test(self, data: dict) -> operations.TestFunctionResponse:
        """
        Test a pipeline's function with a sample input JSON

        Args:
            data: Input JSON

        Returns:
            TestFunctionResponse: Test function response
        """
        request = operations.TestFunctionRequest(
            pipeline_id=self.id,
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
            request_body=data,
        )

        base_res = self._request(
            method="POST",
            endpoint=f"/pipelines/{self.id}/functions/main/test",
            request=request,
        )
        base_res_json = base_res.raw_response.json()
        base_res_json["event_context"] = api.EventContext(
            **base_res_json["event_context"]
        )
        return operations.TestFunctionResponse(
            status_code=base_res.status_code,
            content_type=base_res.content_type,
            raw_response=base_res.raw_response,
            **base_res_json,
        )



# ruff: noqa
# generated by datamodel-codegen:
#   filename:  https://api.glassflow.dev/v1/openapi.yaml
#   version:   0.26.0

from __future__ import annotations
from dataclasses_json import dataclass_json

from dataclasses import dataclass
from enum import Enum
from typing import Any, Dict, List, Optional, Union


@dataclass_json
@dataclass
class Error:
    detail: str


@dataclass_json
@dataclass
class CreateOrganization:
    name: str


@dataclass_json
@dataclass
class Organization(CreateOrganization):
    id: str


@dataclass_json
@dataclass
class OrganizationScope(Organization):
    role: str


OrganizationScopes = List[OrganizationScope]


@dataclass_json
@dataclass
class SignUp:
    access_token: str
    id_token: str


@dataclass_json
@dataclass
class BasePipeline:
    name: str
    space_id: str
    metadata: Dict[str, Any]


class PipelineState(str, Enum):
    running = "running"
    paused = "paused"


@dataclass_json
@dataclass
class FunctionEnvironment:
    name: str
    value: str


FunctionEnvironments = Optional[List[FunctionEnvironment]]


class Kind(str, Enum):
    google_pubsub = "google_pubsub"


@dataclass_json
@dataclass
class Config:
    project_id: str
    subscription_id: str
    credentials_json: str


@dataclass_json
@dataclass
class SourceConnector1:
    kind: Kind
    config: Config


class Kind1(str, Enum):
    amazon_sqs = "amazon_sqs"


@dataclass_json
@dataclass
class Config1:
    queue_url: str
    aws_region: str
    aws_access_key: str
    aws_secret_key: str


@dataclass_json
@dataclass
class SourceConnector2:
    kind: Kind1
    config: Config1


class Kind2(str, Enum):
    postgres = "postgres"


@dataclass_json
@dataclass
class Config2:
    db_host: str
    db_user: str
    db_pass: str
    db_name: str
    replication_slot: str
    db_port: Optional[str] = "5432"
    db_sslmode: Optional[str] = None
    publication: Optional[str] = None
    replication_output_plugin_name: Optional[str] = "wal2json"
    replication_output_plugin_args: Optional[List[str]] = None


@dataclass_json
@dataclass
class SourceConnector3:
    kind: Kind2
    config: Config2


SourceConnector = Optional[Union[SourceConnector1, SourceConnector2, SourceConnector3]]


class Kind3(str, Enum):
    webhook = "webhook"


class Method(str, Enum):
    get = "GET"
    post = "POST"
    put = "PUT"
    patch = "PATCH"
    delete = "DELETE"


@dataclass_json
@dataclass
class Header:
    name: str
    value: str


@dataclass_json
@dataclass
class Config3:
    url: str
    method: Method
    headers: List[Header]


@dataclass_json
@dataclass
class SinkConnector1:
    kind: Kind3
    config: Config3


class Kind4(str, Enum):
    clickhouse = "clickhouse"


@dataclass_json
@dataclass
class Config4:
    addr: str
    database: str
    username: str
    password: str
    table: str


@dataclass_json
@dataclass
class SinkConnector2:
    kind: Kind4
    config: Config4


class Kind5(str, Enum):
    amazon_s3 = "amazon_s3"


@dataclass_json
@dataclass
class Config5:
    s3_bucket: str
    s3_key: str
    aws_region: str
    aws_access_key: str
    aws_secret_key: str


@dataclass_json
@dataclass
class SinkConnector3:
    kind: Kind5
    config: Config5


class Kind6(str, Enum):
    snowflake_cdc_json = "snowflake_cdc_json"


@dataclass_json
@dataclass
class Config6:
    account: str
    warehouse: str
    db_user: str
    db_pass: str
    db_name: str
    db_schema: str
    db_host: Optional[str] = None
    db_port: Optional[str] = "443"
    db_role: Optional[str] = None


@dataclass_json
@dataclass
class SinkConnector4:
    kind: Kind6
    config: Config6


SinkConnector = Optional[
    Union[SinkConnector1, SinkConnector2, SinkConnector3, SinkConnector4]
]


@dataclass_json
@dataclass
class Pipeline(BasePipeline):
    id: str
    created_at: str
    state: PipelineState


@dataclass_json
@dataclass
class SpacePipeline(Pipeline):
    space_name: str


@dataclass_json
@dataclass
class GetDetailedSpacePipeline(SpacePipeline):
    source_connector: SourceConnector
    sink_connector: SinkConnector
    environments: FunctionEnvironments


@dataclass_json
@dataclass
class PipelineFunctionOutput:
    environments: FunctionEnvironments


SpacePipelines = List[SpacePipeline]


@dataclass_json
@dataclass
class CreateSpace:
    name: str


@dataclass_json
@dataclass
class UpdateSpace:
    name: str


@dataclass_json
@dataclass
class Space(CreateSpace):
    id: str
    created_at: str


@dataclass_json
@dataclass
class SpaceScope(Space):
    permission: str


SpaceScopes = List[SpaceScope]


@dataclass_json
@dataclass
class Payload:
    message: str


class SeverityCodeInput(int, Enum):
    integer_100 = 100
    integer_200 = 200
    integer_400 = 400
    integer_500 = 500


SeverityCode = int


@dataclass_json
@dataclass
class CreateAccessToken:
    name: str


@dataclass_json
@dataclass
class AccessToken(CreateAccessToken):
    id: str
    token: str
    created_at: str


AccessTokens = List[AccessToken]


@dataclass_json
@dataclass
class PaginationResponse:
    total_amount: int


@dataclass_json
@dataclass
class SourceFile:
    name: str
    content: str


SourceFiles = List[SourceFile]


@dataclass_json
@dataclass
class EventContext:
    request_id: str
    receive_time: str
    external_id: Optional[str] = None


PersonalAccessToken = str


QueryRangeMatrix = Optional[Any]


@dataclass_json
@dataclass
class Profile:
    id: str
    home_organization: Organization
    name: str
    email: str
    provider: str
    external_settings: Any
    subscriber_id: str


@dataclass_json
@dataclass
class ListOrganizationScopes(PaginationResponse):
    organizations: OrganizationScopes


@dataclass_json
@dataclass
class UpdatePipeline:
    name: str
    transformation_function: Optional[str] = None
    transformation_requirements: Optional[List[str]] = None
    requirements_txt: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    source_connector: Optional[SourceConnector] = None
    sink_connector: Optional[SinkConnector] = None
    environments: Optional[FunctionEnvironments] = None


@dataclass_json
@dataclass
class CreatePipeline(BasePipeline):
    transformation_function: Optional[str] = None
    transformation_requirements: Optional[List[str]] = None
    requirements_txt: Optional[str] = None
    source_connector: Optional[SourceConnector] = None
    sink_connector: Optional[SinkConnector] = None
    environments: Optional[FunctionEnvironments] = None
    state: Optional[PipelineState] = None


@dataclass_json
@dataclass
class ListPipelines(PaginationResponse):
    pipelines: SpacePipelines


@dataclass_json
@dataclass
class ListSpaceScopes(PaginationResponse):
    spaces: SpaceScopes


@dataclass_json
@dataclass
class FunctionLogEntry:
    level: str
    severity_code: SeverityCode
    timestamp: str
    payload: Payload


@dataclass_json
@dataclass
class ListAccessTokens(PaginationResponse):
    access_tokens: AccessTokens


@dataclass_json
@dataclass
class ConsumeInputEvent:
    payload: Any
    event_context: EventContext
    req_id: Optional[str] = None
    receive_time: Optional[str] = None


@dataclass_json
@dataclass
class ConsumeOutputEvent:
    payload: Any
    event_context: EventContext
    status: str
    req_id: Optional[str] = None
    receive_time: Optional[str] = None
    response: Optional[Any] = None
    error_details: Optional[str] = None
    stack_trace: Optional[str] = None


@dataclass_json
@dataclass
class ListPersonalAccessTokens:
    tokens: List[PersonalAccessToken]


@dataclass_json
@dataclass
class PipelineInputQueueRelativeLatencyMetricsResponse:
    input_queue_total_push_events: QueryRangeMatrix
    input_queue_latency: QueryRangeMatrix


FunctionLogs = List[FunctionLogEntry]



from __future__ import annotations

import dataclasses

from .base import BasePipelineDataRequest, BasePipelineManagementRequest


@dataclasses.dataclass
class StatusAccessTokenRequest(BasePipelineDataRequest):
    """Request check the status of an access token

    Attributes:
        pipeline_id: The id of the pipeline
        organization_id: The id of the organization
        x_pipeline_access_token: The access token of the pipeline

    """

    pass


@dataclasses.dataclass
class ListAccessTokensRequest(BasePipelineManagementRequest):
    page_size: int = 50
    page: int = 1



"""Dataclasses for publish event operation"""

from __future__ import annotations

import dataclasses

from .base import BasePipelineDataRequest, BaseResponse


@dataclasses.dataclass
class PublishEventRequestBody:
    pass


@dataclasses.dataclass
class PublishEventRequest(BasePipelineDataRequest):
    """Request to publish an event to a pipeline topic

    Attributes:
        pipeline_id: The id of the pipeline
        organization_id: The id of the organization
        x_pipeline_access_token: The access token of the pipeline
        request_body: The request body / event that should be published to the pipeline
    """

    request_body: dict = dataclasses.field(
        default=None, metadata={"request": {"media_type": "application/json"}}
    )


@dataclasses.dataclass
class PublishEventResponseBody:
    """Message pushed to the pipeline"""


@dataclasses.dataclass
class PublishEventResponse(BaseResponse):
    """Response object for publish event operation

    Attributes:
        content_type: HTTP response content type for this operation
        status_code: HTTP response status code for this operation
        raw_response: Raw HTTP response; suitable for custom response parsing
        object: Response to the publish operation

    """

    object: PublishEventResponseBody | None = dataclasses.field(default=None)



from .utils import (
    generate_metadata_for_query_parameters,
    generate_url,
    get_field_name,
    get_query_params,
    get_req_specific_headers,
    marshal_json,
    match_content_type,
    serialize_request_body,
    unmarshal_json,
)

__all__ = [
    "match_content_type",
    "unmarshal_json",
    "generate_url",
    "serialize_request_body",
    "get_query_params",
    "get_req_specific_headers",
    "get_field_name",
    "marshal_json",
    "generate_metadata_for_query_parameters",
]



from __future__ import annotations

import dataclasses
from enum import Enum

from ...utils import generate_metadata_for_query_parameters
from ..api import (
    CreatePipeline,
    GetDetailedSpacePipeline,
    PipelineState,
    SinkConnector,
    SourceConnector,
    SpacePipeline,
)
from .base import BaseManagementRequest, BasePipelineManagementRequest, BaseResponse


@dataclasses.dataclass
class GetPipelineRequest(BasePipelineManagementRequest):
    pass


@dataclasses.dataclass
class GetPipelineResponse(BaseResponse):
    pipeline: GetDetailedSpacePipeline | None = dataclasses.field(default=None)


@dataclasses.dataclass
class CreatePipelineRequest(BaseManagementRequest, CreatePipeline):
    pass


@dataclasses.dataclass
class UpdatePipelineRequest(BaseManagementRequest):
    name: str | None = dataclasses.field(default=None)
    state: PipelineState | None = dataclasses.field(default=None)
    metadata: dict | None = dataclasses.field(default=None)
    source_connector: SourceConnector | None = dataclasses.field(default=None)
    sink_connector: SinkConnector | None = dataclasses.field(default=None)


@dataclasses.dataclass
class UpdatePipelineResponse(BaseResponse):
    pipeline: GetDetailedSpacePipeline | None = dataclasses.field(default=None)


@dataclasses.dataclass
class CreatePipelineResponse(BaseResponse):
    name: str
    space_id: str
    id: str
    created_at: str
    state: PipelineState
    access_token: str
    metadata: dict | None = dataclasses.field(default=None)


@dataclasses.dataclass
class DeletePipelineRequest(BasePipelineManagementRequest):
    pass


class Order(str, Enum):
    asc = "asc"
    desc = "desc"


@dataclasses.dataclass
class ListPipelinesRequest(BaseManagementRequest):
    space_id: list[str] | None = dataclasses.field(
        default=None,
        metadata=generate_metadata_for_query_parameters("space_id"),
    )
    page_size: int = dataclasses.field(
        default=50,
        metadata=generate_metadata_for_query_parameters("page_size"),
    )
    page: int = dataclasses.field(
        default=1,
        metadata=generate_metadata_for_query_parameters("page"),
    )
    order_by: Order = dataclasses.field(
        default=Order.asc,
        metadata=generate_metadata_for_query_parameters("order_by"),
    )


@dataclasses.dataclass
class ListPipelinesResponse(BaseResponse):
    total_amount: int
    pipelines: list[SpacePipeline]



from __future__ import annotations

import dataclasses

from .base import BasePipelineManagementRequest


@dataclasses.dataclass
class GetArtifactRequest(BasePipelineManagementRequest):
    pass


@dataclasses.dataclass
class PostArtifactRequest(BasePipelineManagementRequest):
    file: str | None = dataclasses.field(
        default=None,
        metadata={
            "multipart_form": {
                "field_name": "file",
            }
        },
    )
    requirementsTxt: str | None = dataclasses.field(default=None)



"""GlassFlow Python Client to interact with GlassFlow API"""

from __future__ import annotations

from .api_client import APIClient
from .models import errors, operations
from .models.api import PipelineState
from .pipeline import Pipeline
from .space import Space


class GlassFlowClient(APIClient):
    """
    GlassFlow Client to interact with GlassFlow API and manage pipelines
    and other resources

    Attributes:
        client: requests.Session object to make HTTP requests to GlassFlow API
        glassflow_config: GlassFlowConfig object to store configuration
        organization_id: Organization ID of the user. If not provided,
            the default organization will be used

    """

    def __init__(
        self, personal_access_token: str = None, organization_id: str = None
    ) -> None:
        """Create a new GlassFlowClient object

        Args:
            personal_access_token: GlassFlow Personal Access Token
            organization_id: Organization ID of the user. If not provided,
                the default organization will be used
        """
        super().__init__()
        self.personal_access_token = personal_access_token
        self.organization_id = organization_id

    def get_pipeline(self, pipeline_id: str) -> Pipeline:
        """Gets a Pipeline object from the GlassFlow API

        Args:
            pipeline_id: UUID of the pipeline

        Returns:
            Pipeline: Pipeline object from the GlassFlow API

        Raises:
            PipelineNotFoundError: Pipeline does not exist
            UnauthorizedError: User does not have permission to perform the
                requested operation
            ClientError: GlassFlow Client Error
        """
        return Pipeline(
            personal_access_token=self.personal_access_token, id=pipeline_id
        ).fetch()

    def create_pipeline(
        self,
        name: str,
        space_id: str,
        transformation_file: str = None,
        requirements: str = None,
        source_kind: str = None,
        source_config: dict = None,
        sink_kind: str = None,
        sink_config: dict = None,
        env_vars: list[dict[str, str]] = None,
        state: PipelineState = "running",
        metadata: dict = None,
    ) -> Pipeline:
        """Creates a new GlassFlow pipeline

        Args:
            name: Name of the pipeline
            space_id: ID of the GlassFlow Space you want to create the pipeline in
            transformation_file: Path to file with transformation function of
                the pipeline.
            requirements: Requirements.txt of the pipeline
            source_kind: Kind of source for the pipeline. If no source is
                provided, the default source will be SDK
            source_config: Configuration of the pipeline's source
            sink_kind: Kind of sink for the pipeline. If no sink is provided,
                the default sink will be SDK
            sink_config: Configuration of the pipeline's sink
            env_vars: Environment variables to pass to the pipeline
            state: State of the pipeline after creation.
                It can be either "running" or "paused"
            metadata: Metadata of the pipeline

        Returns:
            Pipeline: New pipeline

        Raises:
            UnauthorizedError: User does not have permission to perform
                the requested operation
        """
        return Pipeline(
            name=name,
            space_id=space_id,
            transformation_file=transformation_file,
            requirements=requirements,
            source_kind=source_kind,
            source_config=source_config,
            sink_kind=sink_kind,
            sink_config=sink_config,
            env_vars=env_vars,
            state=state,
            metadata=metadata,
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
        ).create()

    def list_pipelines(
        self, space_ids: list[str] | None = None
    ) -> operations.ListPipelinesResponse:
        """
        Lists all pipelines in the GlassFlow API

        Args:
            space_ids: List of Space IDs of the pipelines to list.
                If not specified, all the pipelines will be listed.

        Returns:
            ListPipelinesResponse: Response object with the pipelines listed

        Raises:
            UnauthorizedError: User does not have permission to perform the
                requested operation
        """
        request = operations.ListPipelinesRequest(
            space_id=space_ids,
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
        )
        try:
            res = self._request(
                method="GET",
                endpoint="/pipelines",
                request=request,
            )
            res_json = res.raw_response.json()
        except errors.ClientError as e:
            if e.status_code == 401:
                raise errors.UnauthorizedError(e.raw_response) from e
            else:
                raise e

        return operations.ListPipelinesResponse(
            content_type=res.content_type,
            status_code=res.status_code,
            raw_response=res.raw_response,
            total_amount=res_json["total_amount"],
            pipelines=res_json["pipelines"],
        )

    def list_spaces(self) -> operations.ListSpacesResponse:
        """
        Lists all GlassFlow spaces in the GlassFlow API

        Returns:
            ListSpacesResponse: Response object with the spaces listed

        Raises:
            UnauthorizedError: User does not have permission to perform the
                requested operation
        """
        request = operations.ListSpacesRequest(
            organization_id=self.organization_id,
            personal_access_token=self.personal_access_token,
        )
        try:
            res = self._request(
                method="GET",
                endpoint="/spaces",
                request=request,
            )
            res_json = res.raw_response.json()
        except errors.ClientError as e:
            if e.status_code == 401:
                raise errors.UnauthorizedError(e.raw_response) from e
            else:
                raise e

        return operations.ListSpacesResponse(
            content_type=res.content_type,
            status_code=res.status_code,
            raw_response=res.raw_response,
            total_amount=res_json["total_amount"],
            spaces=res_json["spaces"],
        )

    def create_space(
        self,
        name: str,
    ) -> Space:
        """Creates a new Space

        Args:
            name: Name of the Space

        Returns:
            Space: New space

        Raises:
            UnauthorizedError: User does not have permission to perform
                the requested operation
        """
        return Space(
            name=name,
            personal_access_token=self.personal_access_token,
            organization_id=self.organization_id,
        ).create()
